/*
 *  Copyright (C) 2011 Masaryk University, Faculty of Informatics
 *
 *  Author: Daniel Kopecek <xkopecek@fi.muni.cz>
 *
 *  LICENSE TERMS
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *  3. Neither the name of the Company nor the names of its contributors
 *  may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  This software is provided ``as is'', and any express or implied
 *  warranties, including, but not limited to, the implied warranties of
 *  merchantability and fitness for a particular purpose are disclaimed.
 *  In no event shall the company or contributors be liable for any
 *  direct, indirect, incidental, special, exemplary, or consequential
 *  damages (including, but not limited to, procurement of substitute
 *  goods or services; loss of use, data, or profits; or business
 *  interruption) however caused and on any theory of liability, whether
 *  in contract, strict liability, or tort (including negligence or
 *  otherwise) arising in any way out of the use of this software, even
 *  if advised of the possibility of such damage.
 */
#ifndef HELPERS_H
#define HELPERS_H

/**
 * Memory allocation helpers
 */

/**
 * Allocate storage for a variable of type `T'.
 */
#define alloc_type(T)                           \
        ((T *) malloc(sizeof(T)))

/**
 * Allocate an array of variables of type `T'.
 */
#define alloc_array(T, count)                   \
        ((T *) malloc(sizeof(T) * count))

#define realloc_array(ptr, T, count)            \
        ((T *) realloc(ptr, sizeof(T) * (size_t)(count)))

#define __lsym(s) s ## __LINE__

/**
 * Various useful functions, macros, ...
 */

/**
 * Save errno, execute the block, restore errno.
 */
#define errno_protect \
        for(int __lsym(errno) = errno, __lsym(exec) = 1; __lsym(exec) == 1; __lsym(exec) = 0, errno = __lsym(errno))

/**
 * Output to stderr in debug mode, otherwise do nothing.
 */
#ifndef NDEBUG
# define dP(...) fprintf(stderr, __VA_ARGS__)
#else
# define dP(...) while(0)
#endif

#endif /* HELPERS_H */
