#ifndef ALGORITHM_H
#define ALGORITHM_H

#include <stddef.h>
#include <stdint.h>
#include "libtriplex_private.h"
#include "libtriplex/gff3.h"

/* status variable flags */
#define STAT_NONE         0x00
#define STAT_QUALITY      0x10
#define STAT_MINLEN       0x20
#define STAT_EXPORT       0x40

/* DP rule constants */
#define DP_MATCH        0
#define DP_MISMATCH     1
#define DP_LEFT         2
#define DP_RIGHT        3

#define NUM_TRI_TYPES   8

/* --------------------------------------------------------------------------
 *                          Type definition
 * --------------------------------------------------------------------------
 */

/**
 * Scoring scheme structure.
 * Used for passing scoring scheme to the get_max_score function
 */
typedef struct {
        int pen_isochange;
        int pen_mismatch;
        int pen_insdel;
        int dtwist_limit;
        int score_isostay;
} sc_scheme_t;

/* DP matrix position represented as diagonal and antidiagonal */
typedef struct{
        int     diag;
        int     antidiag;
} t_pos;

/**
 * Data stored for every diagonal
 */
typedef struct{
        t_pos    start;         /**< Position of the first match */
        t_pos    max_score_pos; /**< Maximal score position */

        uint8_t bound;  /**< Type of the triplex bound */
        uint8_t twist;  /**< angle between C1 atoms */
        int8_t  dtwist; /**< Change in angle between subsequent triplets */
        uint8_t dpst;   /**< status (bit 4-7) + dp_rule (bit 0-3, previous position: 0 - match, 1 - mismatch, 2 - left, 3 - right) */
        int16_t score;     /**< Actual score */
        int16_t max_score; /**< Maximal score */

        uint16_t indels;     /**< number of indels */
        uint16_t max_indels; /**< number of indels from start position to max score postition */

} t_diag;

#define DPST_DPMASK 0x0f
#define DPST_STMASK 0xf0

int triplex_search(triplexCTX *ctx, const char *buffer_id, const uint8_t *buffer, size_t buflen, uint64_t offset, int tri_type, gff3_t *result);

#endif /* ALGORITHM_H */
