/*
 *  Copyright (C) 2011 Masaryk University, Faculty of Informatics
 *
 *  Author: Daniel Kopecek <xkopecek@fi.muni.cz>
 *
 *  LICENSE TERMS
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *  3. Neither the name of the Company nor the names of its contributors
 *  may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  This software is provided ``as is'', and any express or implied
 *  warranties, including, but not limited to, the implied warranties of
 *  merchantability and fitness for a particular purpose are disclaimed.
 *  In no event shall the company or contributors be liable for any
 *  direct, indirect, incidental, special, exemplary, or consequential
 *  damages (including, but not limited to, procurement of substitute
 *  goods or services; loss of use, data, or profits; or business
 *  interruption) however caused and on any theory of liability, whether
 *  in contract, strict liability, or tort (including negligence or
 *  otherwise) arising in any way out of the use of this software, even
 *  if advised of the possibility of such damage.
 */
#ifndef LIBTRIPLEX_PRIVATE
#define LIBTRIPLEX_PRIVATE

#include <config.h>
#include <stdint.h>
#include <stddef.h>

#include <libfasta/fasta.h>

#include "libtriplex/libtriplex.h"

typedef struct {
        char    *name; /**< parameter name */
        char    *description; /**< parameter description */
        uint32_t flags; /**< parameter flags: type, array, NULL, ... */

        union {
                int    _int;
                double _float;
                char  *_string;

                struct {
                        size_t  count;
                        union {
                                int     *_int;
                                double  *_float;
                                char   **_string;
                        } values;
                } _array;
        } value;
} triplexParam;

#define TRIPLEX_PARAM_TYPEMASK 0x0000000f
#define TRIPLEX_PARAM_FLAGMASK 0xfffffff0

/**
 * flags
 */
#define TRIPLEX_PFLAG_PUB1     0x00000010 /**< the value of the parameter is or is expected to be an array of values */
#define TRIPLEX_PFLAG_NULL     0x00000020 /**< the value of the parameter is NULL/not set */
#define TRIPLEX_PFLAG_FREE     0x00000040 /**< storage for the value was dynamically allocated */

struct triplexCTX {
        uint32_t       flags;

        triplexParam  *params;
        size_t         pcount;
};

#endif /* LIBTRIPLEX_PRIVATE */
